from datetime import datetime
from django.dispatch import receiver
from django.utils.timezone import make_aware
from .signals import subscription_created
from .models import PaddlePlan, PaddleSubscription
from .utils import get_account_by_passthrough


@receiver(subscription_created)
def subscription_created_receiver(**kwargs):
    payload = kwargs['payload']

    account = get_account_by_passthrough(payload['passthrough'])

    try:
        plan = PaddlePlan.objects.get(id=payload['subscription_plan_id'])
    except PaddlePlan.DoesNotExist:
        plan = None

    PaddleSubscription.objects.create(
        id=payload['subscription_id'],
        account=account,
        plan=plan,
        user_id=payload['user_id'],
        user_email=payload['email'],
        marketing_consent=payload['marketing_consent'] == True,
        update_url=payload['update_url'],
        cancel_url=payload['cancel_url'],
        state=payload['status'],
        signup_date=make_aware(datetime.strptime(payload['event_time'], '%Y-%m-%d %H:%M:%S'))
    )



from datetime import datetime

from django.db import models
from django.utils.timezone import make_aware
from django_paddle.client import PaddleClient
from django_paddle.utils import get_account_model, get_account_by_passthrough



pc = PaddleClient()


class PaddlePlan(models.Model):
    id = models.PositiveIntegerField(
        primary_key=True,
        unique=True
    )
    name = models.CharField(max_length=255)
    billing_type = models.CharField(max_length=255)
    billing_period = models.PositiveIntegerField()
    trial_days = models.PositiveIntegerField()

    def initial_price_in(self, currency):
        return self.initial_prices.get(currency=currency).amount

    def recurring_price_in(self, currency):
        return self.recurring_prices.get(currency=currency).amount

    @staticmethod
    def sync():
        for plan in pc.plans_list():
            plan_obj, _ = PaddlePlan.objects.update_or_create(
                id=plan['id'],
                defaults={
                    'name': plan['name'],
                    'billing_type': plan['billing_type'],
                    'billing_period': plan['billing_period'],
                    'trial_days': plan['trial_days']
                }
            )
            for currency, amount in plan['initial_price'].items():
                plan_obj.initial_prices.update_or_create(
                    currency=currency,
                    defaults={
                        'amount': amount
                    }
                )
            for currency, amount in plan['recurring_price'].items():
                plan_obj.recurring_prices.update_or_create(
                    currency=currency,
                    defaults={
                        'amount': amount
                    }
                )


class PaddlePrice(models.Model):
    plan = models.ForeignKey(
        to=PaddlePlan,
        on_delete=models.CASCADE,
    )
    currency = models.CharField(max_length=255)
    amount = models.CharField(max_length=255)

    class Meta:
        abstract = True
        unique_together = ['plan', 'currency']


class PaddleInitialPrice(PaddlePrice):
    class Meta:
        default_related_name = 'initial_prices'


class PaddleRecurringPrice(PaddlePrice):

    class Meta:
        default_related_name = 'recurring_prices'


class PaddleSubscription(models.Model):
    id = models.PositiveIntegerField(
        primary_key=True,
        unique=True
    )
    account = models.ForeignKey(
        to=get_account_model(),
        null=True,
        on_delete=models.SET_NULL,
        related_name='subscriptions'
    )
    plan = models.ForeignKey(
        to=PaddlePlan,
        null=True,
        on_delete=models.SET_NULL,
        related_name='subscriptions'
    )
    user_id = models.PositiveIntegerField()
    user_email = models.EmailField()
    marketing_consent = models.BooleanField()
    update_url = models.CharField(max_length=255)
    cancel_url = models.CharField(max_length=255)
    state = models.CharField(max_length=255)
    signup_date = models.DateTimeField()

    def cancel(self):
        pc.subscriptions_cancel(self.id)
        self.state = 'deleted'
        self.save()

    def pause(self):
        pc.subscriptions_pause(subscription_id=self.id)
        self.state = 'paused'
        self.save()

    def unpause(self):
        pc.subscriptions_unpause(subscription_id=self.id)
        self.state = 'active'
        self.save()

    @staticmethod
    def sync():
        for sub in pc.subscriptions_list():

            transaction = pc.transactions_list(entity='subscription', id=sub['subscription_id'])[0]
            account = get_account_by_passthrough(transaction['passthrough'])

            try:
                plan = PaddlePlan.objects.get(id=sub['plan_id'])
            except PaddlePlan.DoesNotExist:
                plan = None

            defaults = {
                    'user_id': sub['user_id'],
                    'user_email': sub['user_email'],
                    'marketing_consent': sub['marketing_consent'],
                    'update_url': sub['update_url'],
                    'cancel_url': sub['cancel_url'],
                    'state': sub['state'],
                    'signup_date': make_aware(datetime.strptime(sub['signup_date'], '%Y-%m-%d %H:%M:%S'))
            }

            if plan:
                defaults['plan'] = plan

            if account:
                defaults['account'] = account

            PaddleSubscription.objects.update_or_create(
                id=sub['subscription_id'],
                defaults=defaults
            )


class PaddlePayment(models.Model):
    id = models.PositiveIntegerField(
        unique=True,
        primary_key=True
    )
    subscription = models.ForeignKey(
        to=PaddleSubscription,
        on_delete=models.CASCADE,
        related_name='payments'
    )
    amount = models.PositiveIntegerField()
    currency = models.CharField(max_length=255)
    payout_date = models.CharField(max_length=255)
    is_paid = models.BooleanField()
    is_one_off_charge = models.BooleanField()
    receipt_url = models.CharField(max_length=255)



from setuptools import setup, find_packages


setup(
    name='django-paddle',
    version='0.0.5',
    packages=find_packages(),
    description='Django models for integrating Paddle.com subscriptions',
    url='https://github.com/kennell/django-paddle',
    author='Kevin Kennell',
    author_email='kevin@kennell.de',
    install_requires=[
        'django',
        'requests',
        'cryptography',
        'phpserialize'
    ],
    extras_require={
        'dev': [
            'responses',
        ]
    }
)



# Generated by Django 3.0.5 on 2020-05-03 18:51

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PaddlePlan',
            fields=[
                ('id', models.PositiveIntegerField(primary_key=True, serialize=False, unique=True)),
                ('name', models.CharField(max_length=255)),
                ('billing_type', models.CharField(max_length=255)),
                ('billing_period', models.PositiveIntegerField()),
                ('trial_days', models.PositiveIntegerField()),
            ],
        ),
        migrations.CreateModel(
            name='PaddleSubscription',
            fields=[
                ('id', models.PositiveIntegerField(primary_key=True, serialize=False, unique=True)),
                ('user_id', models.PositiveIntegerField()),
                ('user_email', models.EmailField(max_length=254)),
                ('marketing_consent', models.BooleanField()),
                ('update_url', models.CharField(max_length=255)),
                ('cancel_url', models.CharField(max_length=255)),
                ('state', models.CharField(max_length=255)),
                ('signup_date', models.DateTimeField()),
                ('account', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscriptions', to=settings.AUTH_USER_MODEL)),
                ('plan', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='subscriptions', to='django_paddle.PaddlePlan')),
            ],
        ),
        migrations.CreateModel(
            name='PaddleRecurringPrice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('currency', models.CharField(max_length=255)),
                ('amount', models.CharField(max_length=255)),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='recurring_prices', to='django_paddle.PaddlePlan')),
            ],
            options={
                'default_related_name': 'recurring_prices',
            },
        ),
        migrations.CreateModel(
            name='PaddlePayment',
            fields=[
                ('id', models.PositiveIntegerField(primary_key=True, serialize=False, unique=True)),
                ('amount', models.PositiveIntegerField()),
                ('currency', models.CharField(max_length=255)),
                ('payout_date', models.CharField(max_length=255)),
                ('is_paid', models.BooleanField()),
                ('is_one_off_charge', models.BooleanField()),
                ('receipt_url', models.CharField(max_length=255)),
                ('subscription', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='payments', to='django_paddle.PaddleSubscription')),
            ],
        ),
        migrations.CreateModel(
            name='PaddleInitialPrice',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('currency', models.CharField(max_length=255)),
                ('amount', models.CharField(max_length=255)),
                ('plan', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='initial_prices', to='django_paddle.PaddlePlan')),
            ],
            options={
                'default_related_name': 'initial_prices',
            },
        ),
    ]
